@@ -471,7 +471,8 @@ gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
   if (GST_IS_DISCOVERER_AUDIO_INFO (info))
     return "audio";
   if (GST_IS_DISCOVERER_VIDEO_INFO (info)) {
-    if (gst_discoverer_video_info_get_is_image (info))
+    if (gst_discoverer_video_info_is_image ((GstDiscovererVideoInfo *)
+            info))
       return "video(image)";
     else
       return "video";
@@ -481,10 +482,11 @@ gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
 
 /* ACCESSORS */
 
-#define GENERIC_ACCESSOR_CODE(parent, casttype, parenttype, parentgtype, fieldname, type, failval) \
-  type parent##_get_##fieldname(parenttype info) {			\
+
+#define GENERIC_ACCESSOR_CODE(parent, parenttype, parentgtype, fieldname, type, failval) \
+  type parent##_get_##fieldname(const parenttype info) {			\
     g_return_val_if_fail(G_TYPE_CHECK_INSTANCE_TYPE((info), parentgtype), failval); \
-    return ((casttype)info)->fieldname;				\
+    return (info)->fieldname;				\
   }
 
 /**
@@ -539,7 +541,9 @@ gst_discoverer_stream_info_get_caps (GstDiscovererStreamInfo * info)
 {
   g_return_val_if_fail (GST_IS_DISCOVERER_STREAM_INFO (info), NULL);
 
-  return gst_caps_ref (info->caps);
+  if (info->caps)
+    return gst_caps_ref (info->caps);
+  return NULL;
 }
 
 /**
@@ -610,13 +614,12 @@ gst_discoverer_container_info_get_streams (GstDiscovererStreamInfo * info)
 
 #define AUDIO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
   GENERIC_ACCESSOR_CODE(gst_discoverer_audio_info, GstDiscovererAudioInfo*, \
-			GstDiscovererStreamInfo*,			\
 			GST_TYPE_DISCOVERER_AUDIO_INFO,		\
 			fieldname, type, failval)
 
 /**
  * gst_discoverer_audio_info_get_channels:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererAudioInfo
  *
  * Returns: the number of channels in the stream.
  *
@@ -627,7 +630,7 @@ AUDIO_INFO_ACCESSOR_CODE (channels, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_sample_rate:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererAudioInfo
  *
  * Returns: the sample rate of the stream in Hertz.
  *
@@ -638,7 +641,7 @@ AUDIO_INFO_ACCESSOR_CODE (sample_rate, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_depth:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererAudioInfo
  *
  * Returns: the number of bits used per sample in each channel.
  *
@@ -649,7 +652,7 @@ AUDIO_INFO_ACCESSOR_CODE (depth, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_bitrate:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererAudioInfo
  *
  * Returns: the average or nominal bitrate of the stream in bits/second.
  *
@@ -660,7 +663,7 @@ AUDIO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_max_bitrate:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererAudioInfo
  *
  * Returns: the maximum bitrate of the stream in bits/second.
  *
@@ -673,13 +676,12 @@ AUDIO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
 
 #define VIDEO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
   GENERIC_ACCESSOR_CODE(gst_discoverer_video_info, GstDiscovererVideoInfo*, \
-			GstDiscovererStreamInfo*,			\
 			GST_TYPE_DISCOVERER_VIDEO_INFO,			\
 			fieldname, type, failval)
 
 /**
  * gst_discoverer_video_info_get_width:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the width of the video stream in pixels.
  *
@@ -690,7 +692,7 @@ VIDEO_INFO_ACCESSOR_CODE (width, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_height:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the height of the video stream in pixels.
  *
@@ -701,7 +703,7 @@ VIDEO_INFO_ACCESSOR_CODE (height, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_depth:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the depth in bits of the video stream (only relevant for
  * video streams).
@@ -713,7 +715,7 @@ VIDEO_INFO_ACCESSOR_CODE (depth, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_framerate_num:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the framerate of the video stream (numerator).
  *
@@ -724,7 +726,7 @@ VIDEO_INFO_ACCESSOR_CODE (framerate_num, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_framerate_denom:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the framerate of the video stream (denominator).
  *
@@ -735,7 +737,7 @@ VIDEO_INFO_ACCESSOR_CODE (framerate_denom, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_par_num:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the Pixel Aspect Ratio (PAR) of the video stream (numerator).
  *
@@ -746,7 +748,7 @@ VIDEO_INFO_ACCESSOR_CODE (par_num, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_par_denom:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the Pixel Aspect Ratio (PAR) of the video stream (denominator).
  *
@@ -756,19 +758,24 @@ VIDEO_INFO_ACCESSOR_CODE (par_num, guint, 0);
 VIDEO_INFO_ACCESSOR_CODE (par_denom, guint, 0);
 
 /**
- * gst_discoverer_video_info_get_interlaced:
- * @info: a #GstDiscovererStreamInfo
+ * gst_discoverer_video_info_is_interlaced:
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: %TRUE if the stream is interlaced, else %FALSE.
  *
  * Since 0.10.31
  */
+gboolean
+gst_discoverer_video_info_is_interlaced (const GstDiscovererVideoInfo * info)
+{
+  g_return_val_if_fail (GST_IS_DISCOVERER_VIDEO_INFO (info), FALSE);
 
-VIDEO_INFO_ACCESSOR_CODE (interlaced, gboolean, FALSE);
+  return info->interlaced;
+}
 
 /**
  * gst_discoverer_video_info_get_bitrate:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the average or nominal bitrate of the video stream in bits/second.
  *
@@ -779,7 +786,7 @@ VIDEO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_max_bitrate:
- * @info: a #GstDiscovererStreamInfo
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: the maximum bitrate of the video stream in bits/second.
  *
@@ -789,22 +796,26 @@ VIDEO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
 VIDEO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
 
 /**
- * gst_discoverer_video_info_get_is_image:
- * @info: a #GstDiscovererStreamInfo
+ * gst_discoverer_video_info_is_image:
+ * @info: a #GstDiscovererVideoInfo
  *
  * Returns: #TRUE if the video stream corresponds to an image (i.e. only contains
  * one frame).
  *
  * Since 0.10.31
  */
+gboolean
+gst_discoverer_video_info_is_image (const GstDiscovererVideoInfo * info)
+{
+  g_return_val_if_fail (GST_IS_DISCOVERER_VIDEO_INFO (info), FALSE);
 
-VIDEO_INFO_ACCESSOR_CODE (is_image, gboolean, FALSE);
+  return info->is_image;
+}
 
 /* GstDiscovererInfo */
 
 #define DISCOVERER_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
   GENERIC_ACCESSOR_CODE(gst_discoverer_info, GstDiscovererInfo*,	\
-			GstDiscovererInfo*,				\
 			GST_TYPE_DISCOVERER_INFO,			\
 			fieldname, type, failval)
 
