@@ -18,6 +18,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-tcpclientsrc
+ * @see_also: #tcpclientsink
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * # server:
+ * nc -l -p 3000
+ * # client:
+ * gst-launch tcpclientsrc protocol=none port=3000 ! fdsink fd=2
+ * ]| everything you type in the server is shown on the client
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,12 +52,6 @@ GST_DEBUG_CATEGORY_STATIC (tcpclientsrc_debug);
 #define MAX_READ_SIZE                   4 * 1024
 
 
-static const GstElementDetails gst_tcp_client_src_details =
-GST_ELEMENT_DETAILS ("TCP client source",
-    "Source/Network",
-    "Receive data as a client over the network via TCP",
-    "Thomas Vander Stichele <thomas at apestaart dot org>");
-
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -72,6 +80,7 @@ static GstFlowReturn gst_tcp_client_src_create (GstPushSrc * psrc,
 static gboolean gst_tcp_client_src_stop (GstBaseSrc * bsrc);
 static gboolean gst_tcp_client_src_start (GstBaseSrc * bsrc);
 static gboolean gst_tcp_client_src_unlock (GstBaseSrc * bsrc);
+static gboolean gst_tcp_client_src_unlock_stop (GstBaseSrc * bsrc);
 
 static void gst_tcp_client_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -87,7 +96,10 @@ gst_tcp_client_src_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&srctemplate));
 
-  gst_element_class_set_details (element_class, &gst_tcp_client_src_details);
+  gst_element_class_set_details_simple (element_class,
+      "TCP client source", "Source/Network",
+      "Receive data as a client over the network via TCP",
+      "Thomas Vander Stichele <thomas at apestaart dot org>");
 }
 
 static void
@@ -122,6 +134,7 @@ gst_tcp_client_src_class_init (GstTCPClientSrcClass * klass)
   gstbasesrc_class->start = gst_tcp_client_src_start;
   gstbasesrc_class->stop = gst_tcp_client_src_stop;
   gstbasesrc_class->unlock = gst_tcp_client_src_unlock;
+  gstbasesrc_class->unlock_stop = gst_tcp_client_src_unlock_stop;
 
   gstpush_src_class->create = gst_tcp_client_src_create;
 
@@ -138,8 +151,6 @@ gst_tcp_client_src_init (GstTCPClientSrc * this, GstTCPClientSrcClass * g_class)
   this->protocol = GST_TCP_PROTOCOL_NONE;
   this->caps = NULL;
 
-  gst_base_src_set_live (GST_BASE_SRC (this), TRUE);
-
   GST_OBJECT_FLAG_UNSET (this, GST_TCP_CLIENT_SRC_OPEN);
 }
 
@@ -411,7 +422,20 @@ gst_tcp_client_src_unlock (GstBaseSrc * bsrc)
 {
   GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
 
+  GST_DEBUG_OBJECT (src, "set to flushing");
   gst_poll_set_flushing (src->fdset, TRUE);
 
   return TRUE;
 }
+
+/* will be called only between calls to start() and stop() */
+static gboolean
+gst_tcp_client_src_unlock_stop (GstBaseSrc * bsrc)
+{
+  GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
+
+  GST_DEBUG_OBJECT (src, "unset flushing");
+  gst_poll_set_flushing (src->fdset, FALSE);
+
+  return TRUE;
+}
