@@ -50,6 +50,13 @@
 GST_DEBUG_CATEGORY_STATIC (seek_debug);
 #define GST_CAT_DEFAULT (seek_debug)
 
+#if (!GTK_CHECK_VERSION(2, 23, 0) || GTK_CHECK_VERSION(2, 90, 0)) && !GTK_CHECK_VERSION(2, 91, 1)
+#define gtk_combo_box_text_new gtk_combo_box_new_text
+#define gtk_combo_box_text_append_text gtk_combo_box_append_text
+#define gtk_combo_box_text_remove gtk_combo_box_remove_text
+#define GTK_COMBO_BOX_TEXT GTK_COMBO_BOX
+#endif
+
 #if !GTK_CHECK_VERSION (2, 17, 7)
 static void
 gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
@@ -1788,11 +1795,11 @@ clear_streams (GstElement * pipeline)
 
   /* remove previous info */
   for (i = 0; i < n_video; i++)
-    gtk_combo_box_remove_text (GTK_COMBO_BOX (video_combo), 0);
+    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (video_combo), 0);
   for (i = 0; i < n_audio; i++)
-    gtk_combo_box_remove_text (GTK_COMBO_BOX (audio_combo), 0);
+    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (audio_combo), 0);
   for (i = 0; i < n_text; i++)
-    gtk_combo_box_remove_text (GTK_COMBO_BOX (text_combo), 0);
+    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (text_combo), 0);
 
   n_audio = n_video = n_text = 0;
   gtk_widget_set_sensitive (video_combo, FALSE);
@@ -1833,7 +1840,7 @@ update_streams (GstPipeline * pipeline)
       }
       /* find good name for the label */
       name = g_strdup_printf ("video %d", i + 1);
-      gtk_combo_box_append_text (GTK_COMBO_BOX (video_combo), name);
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (video_combo), name);
       g_free (name);
     }
     state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (video_checkbox));
@@ -1850,7 +1857,7 @@ update_streams (GstPipeline * pipeline)
       }
       /* find good name for the label */
       name = g_strdup_printf ("audio %d", i + 1);
-      gtk_combo_box_append_text (GTK_COMBO_BOX (audio_combo), name);
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (audio_combo), name);
       g_free (name);
     }
     state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (audio_checkbox));
@@ -1879,7 +1886,7 @@ update_streams (GstPipeline * pipeline)
       if (name == NULL)
         name = g_strdup_printf ("text %d", i + 1);
 
-      gtk_combo_box_append_text (GTK_COMBO_BOX (text_combo), name);
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (text_combo), name);
       g_free (name);
     }
     state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (text_checkbox));
@@ -1955,7 +1962,7 @@ init_visualization_features (void)
     name = gst_element_factory_get_longname (entry.factory);
 
     g_array_append_val (vis_entries, entry);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (vis_combo), name);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (vis_combo), name);
   }
   gtk_combo_box_set_active (GTK_COMBO_BOX (vis_combo), 0);
 }
@@ -2451,7 +2458,7 @@ bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * data)
      * shouldn't be done from a non-GUI thread without explicit locking).  */
     g_assert (embed_xid != 0);
 
-    gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (element), embed_xid);
+    gst_x_overlay_set_window_handle (GST_X_OVERLAY (element), embed_xid);
   }
   return GST_BUS_PASS;
 }
@@ -2463,11 +2470,14 @@ handle_expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer data)
   if (state < GST_STATE_PAUSED) {
     GtkAllocation allocation;
     GdkWindow *window = gtk_widget_get_window (widget);
-    GtkStyle *style = gtk_widget_get_style (widget);
+    cairo_t *cr;
 
     gtk_widget_get_allocation (widget, &allocation);
-    gdk_draw_rectangle (window, style->black_gc, TRUE, 0, 0,
-        allocation.width, allocation.height);
+    cr = gdk_cairo_create (window);
+    cairo_set_source_rgb (cr, 0, 0, 0);
+    cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
+    cairo_fill (cr);
+    cairo_destroy (cr);
   }
   return FALSE;
 }
@@ -2793,9 +2803,11 @@ main (int argc, char **argv)
     step = gtk_expander_new ("step options");
     hbox = gtk_hbox_new (FALSE, 0);
 
-    format_combo = gtk_combo_box_new_text ();
-    gtk_combo_box_append_text (GTK_COMBO_BOX (format_combo), "frames");
-    gtk_combo_box_append_text (GTK_COMBO_BOX (format_combo), "time (ms)");
+    format_combo = gtk_combo_box_text_new ();
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (format_combo),
+        "frames");
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (format_combo),
+        "time (ms)");
     gtk_combo_box_set_active (GTK_COMBO_BOX (format_combo), 0);
     gtk_box_pack_start (GTK_BOX (hbox), format_combo, FALSE, FALSE, 2);
 
@@ -2861,9 +2873,9 @@ main (int argc, char **argv)
   if (pipeline_type == 16) {
     /* the playbin2 panel controls for the video/audio/subtitle tracks */
     panel = gtk_hbox_new (FALSE, 0);
-    video_combo = gtk_combo_box_new_text ();
-    audio_combo = gtk_combo_box_new_text ();
-    text_combo = gtk_combo_box_new_text ();
+    video_combo = gtk_combo_box_text_new ();
+    audio_combo = gtk_combo_box_text_new ();
+    text_combo = gtk_combo_box_text_new ();
     gtk_widget_set_sensitive (video_combo, FALSE);
     gtk_widget_set_sensitive (audio_combo, FALSE);
     gtk_widget_set_sensitive (text_combo, FALSE);
@@ -2927,7 +2939,7 @@ main (int argc, char **argv)
         "save a screenshot .png in the current directory");
     g_signal_connect (G_OBJECT (shot_button), "clicked", G_CALLBACK (shot_cb),
         pipeline);
-    vis_combo = gtk_combo_box_new_text ();
+    vis_combo = gtk_combo_box_text_new ();
     g_signal_connect (G_OBJECT (vis_combo), "changed",
         G_CALLBACK (vis_combo_cb), pipeline);
     gtk_widget_set_sensitive (vis_combo, FALSE);
@@ -3021,6 +3033,7 @@ main (int argc, char **argv)
 
   {
     GIOChannel *js_watch = g_io_channel_unix_new (js_fd);
+    g_io_channel_set_encoding (js_watch, NULL, NULL);
     g_io_add_watch (js_watch, G_IO_IN, read_joystick, NULL);
   }
 
